MODULE sbcwave
   !!======================================================================
   !!                       ***  MODULE  sbcwave  ***
   !! Wave module 
   !!======================================================================
   !! History :  3.3  !   2011-09  (Adani M)  Original code: Drag Coefficient 
   !!         :  3.4  !   2012-10  (Adani M)                 Stokes Drift 
   !!            3.6  !   2014-09  (Clementi E, Oddo P)New Stokes Drift Computation
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   sbc_wave      : wave data from wave model in netcdf files 
   !!----------------------------------------------------------------------
   USE oce            ! 
   USE sbc_oce	      ! Surface boundary condition: ocean fields
   USE bdy_oce        !
   USE domvvl         !
   !
   USE iom            ! I/O manager library
   USE in_out_manager ! I/O manager
   USE lib_mpp        ! distribued memory computing library
   USE fldread	      ! read input fields
   USE wrk_nemo       !
   USE phycst         ! physical constants 

   IMPLICIT NONE
   PRIVATE

   PUBLIC   sbc_wave    ! routine called in sbcmod
   
   INTEGER , PARAMETER ::   jpfld  = 4           ! number of files to read for stokes drift
   INTEGER , PARAMETER ::   jp_usd = 1           ! index of stokes drift  (i-component) (m/s)    at T-point
   INTEGER , PARAMETER ::   jp_vsd = 2           ! index of stokes drift  (j-component) (m/s)    at T-point
   INTEGER , PARAMETER ::   jp_swh = 3           ! index of significant wave hight      (m)      at T-point
   INTEGER , PARAMETER ::   jp_wmp = 4           ! index of mean wave period            (s)      at T-point

   TYPE(FLD), ALLOCATABLE, DIMENSION(:)  :: sf_cd    ! structure of input fields (file informations, fields read) Drag Coefficient
   TYPE(FLD), ALLOCATABLE, DIMENSION(:)  :: sf_sd    ! structure of input fields (file informations, fields read) Stokes Drift
   TYPE(FLD), ALLOCATABLE, DIMENSION(:)  :: sf_wn    ! structure of input fields (file informations, fields read) wave number for Qiao
   TYPE(FLD), ALLOCATABLE, DIMENSION(:)  :: sf_tauoc ! structure of input fields (file informations, fields read) normalized wave stress into the ocean
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:)       :: cdn_wave 
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:)       :: swh,wmp, wnum
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:)       :: tauoc_wave
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:)       :: tsd2d
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:)       :: usd2d, vsd2d
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:,:)     :: usd3d, vsd3d, wsd3d 
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:,:)     :: usd3dt, vsd3dt
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:)       :: zusd2dt, zvsd2dt

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.7 , NEMO Consortium (2014) 
   !! $Id$
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE sbc_wave( kt )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE sbc_wave  ***
      !!
      !! ** Purpose :   read wave parameters from wave model  in netcdf files.
      !!
      !! ** Method  : - Read namelist namsbc_wave
      !!              - Read Cd_n10 fields in netcdf files 
      !!              - Read stokes drift 2d in netcdf files 
      !!              - Read wave number in netcdf files 
      !!              - Compute 3d stokes drift using Breivik et al.,2014
      !!                formulation
      !! ** action  
      !!---------------------------------------------------------------------
      USE zdf_oce,  ONLY : ln_zdfqiao

      INTEGER, INTENT( in  ) ::   kt       ! ocean time step
      !
      INTEGER                ::   ierror   ! return error code
      INTEGER                ::   ifpr, jj,ji,jk 
      INTEGER                ::   ios      ! Local integer output status for namelist read
      !
      CHARACTER(len=100)     ::  cn_dir                          ! Root directory for location of drag coefficient files
      REAL(wp)                       ::  ztransp, zsp0, zk, zus, zvs
      REAL(wp), DIMENSION(jpi,jpj)   ::  zfac 
      REAL(wp), DIMENSION(:,:,:), POINTER :: ze3hdiv   ! 3D workspace
      TYPE(FLD_N), DIMENSION(jpfld) ::   slf_i     ! array of namelist informations on the fields to read
      TYPE(FLD_N)            ::  sn_cdg, sn_usd, sn_vsd,  &
                             &   sn_swh, sn_wmp, sn_wnum, sn_tauoc      ! informations about the fields to be read
      !!
      NAMELIST/namsbc_wave/  sn_cdg, cn_dir, sn_usd, sn_vsd, sn_swh, sn_wmp, sn_wnum, sn_tauoc
      !!---------------------------------------------------------------------
      !
      !                                         ! -------------------- !
      IF( kt == nit000 ) THEN                   ! First call kt=nit000 !
         !                                      ! -------------------- !
         REWIND( numnam_ref )              ! Namelist namsbc_wave in reference namelist : File for drag coeff. from wave model
         READ  ( numnam_ref, namsbc_wave, IOSTAT = ios, ERR = 901)
901      IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_wave in reference namelist', lwp )

         REWIND( numnam_cfg )              ! Namelist namsbc_wave in configuration namelist : File for drag coeff. from wave model
         READ  ( numnam_cfg, namsbc_wave, IOSTAT = ios, ERR = 902 )
902      IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_wave in configuration namelist', lwp )
         IF(lwm) WRITE ( numond, namsbc_wave )
         !
         IF ( ln_cdgw ) THEN
            ALLOCATE( sf_cd(1), STAT=ierror )           !* allocate and fill sf_wave with sn_cdg
            IF( ierror > 0 )   CALL ctl_stop( 'STOP', 'sbc_wave: unable to allocate sf_wave structure' )
            !
                                   ALLOCATE( sf_cd(1)%fnow(jpi,jpj,1)   )
            IF( sn_cdg%ln_tint )   ALLOCATE( sf_cd(1)%fdta(jpi,jpj,1,2) )
            CALL fld_fill( sf_cd, (/ sn_cdg /), cn_dir, 'sbc_wave', 'Wave module ', 'namsbc_wave' )
            ALLOCATE( cdn_wave(jpi,jpj) )
            cdn_wave(:,:) = 0.0
         ENDIF

         IF ( ln_tauoc ) THEN
            ALLOCATE( sf_tauoc(1), STAT=ierror )           !* allocate and fill sf_wave with sn_tauoc
            IF( ierror > 0 )   CALL ctl_stop( 'STOP', 'sbc_wave: unable to allocate sf_wave structure' )
            !
                                   ALLOCATE( sf_tauoc(1)%fnow(jpi,jpj,1)   )
            IF( sn_tauoc%ln_tint )   ALLOCATE( sf_tauoc(1)%fdta(jpi,jpj,1,2) )
            CALL fld_fill( sf_tauoc, (/ sn_tauoc /), cn_dir, 'sbc_wave', 'Wave module', 'namsbc_wave' )
            ALLOCATE( tauoc_wave(jpi,jpj) )
            tauoc_wave(:,:) = 0.0
        ENDIF

         IF ( ln_sdw ) THEN
            slf_i(jp_usd) = sn_usd ; slf_i(jp_vsd) = sn_vsd;
            slf_i(jp_swh) = sn_swh ; slf_i(jp_wmp) = sn_wmp;
            ALLOCATE( sf_sd(jpfld), STAT=ierror )           !* allocate and fill sf_sd with stokes drift
            IF( ierror > 0 )   CALL ctl_stop( 'STOP', 'sbc_wave: unable to allocate sf_wave structure' )
            !
            DO ifpr= 1, jpfld
               ALLOCATE( sf_sd(ifpr)%fnow(jpi,jpj,1) )
               IF( slf_i(ifpr)%ln_tint )   ALLOCATE( sf_sd(ifpr)%fdta(jpi,jpj,1,2) )
            END DO

            CALL fld_fill( sf_sd, slf_i, cn_dir, 'sbc_wave', 'Wave module ', 'namsbc_wave' )
            ALLOCATE( usd2d(jpi,jpj),vsd2d(jpi,jpj) )
            ALLOCATE( usd3d(jpi,jpj,jpk),vsd3d(jpi,jpj,jpk),wsd3d(jpi,jpj,jpk) )
            ALLOCATE( usd3dt(jpi,jpj,jpk),vsd3dt(jpi,jpj,jpk) )
            ALLOCATE( swh(jpi,jpj), wmp(jpi,jpj) )
            ALLOCATE( zusd2dt(jpi,jpj), zvsd2dt(jpi,jpj) )
            usd3d(:,:,:) = 0._wp   ;   usd2d(:,:) = 0._wp   ;   
            vsd3d(:,:,:) = 0._wp   ;   vsd2d(:,:) = 0._wp   ; 
            wsd3d(:,:,:) = 0._wp   ;
            usd3dt(:,:,:) = 0._wp  ;   vsd3dt(:,:,:) = 0._wp   ;
            swh  (:,:)   = 0._wp   ;    wmp (:,:) = 0._wp ;
            IF ( ln_zdfqiao ) THEN     !==  Vertical mixing enhancement using Qiao,2010  ==!
               ALLOCATE( sf_wn(1), STAT=ierror )           !* allocate and fill sf_wave with sn_wnum
               IF( ierror > 0 )   CALL ctl_stop( 'STOP', 'sbc_wave: unable toallocate sf_wave structure' )
                                      ALLOCATE( sf_wn(1)%fnow(jpi,jpj,1)   )
               IF( sn_wnum%ln_tint )  ALLOCATE( sf_wn(1)%fdta(jpi,jpj,1,2) )
               CALL fld_fill( sf_wn, (/ sn_wnum /), cn_dir, 'sbc_wave', 'Wave module', 'namsbc_wave' )
               ALLOCATE( wnum(jpi,jpj),tsd2d(jpi,jpj) )
               wnum(:,:) = 0._wp ; tsd2d(:,:) = 0._wp
            ENDIF
         ENDIF
      ENDIF
      !
      IF ( ln_cdgw ) THEN              !==  Neutral drag coefficient  ==!
         CALL fld_read( kt, nn_fsbc, sf_cd )      ! read from external forcing
         cdn_wave(:,:) = sf_cd(1)%fnow(:,:,1)
      ENDIF

      IF ( ln_tauoc ) THEN             !==  Wave induced stress  ==!
         CALL fld_read( kt, nn_fsbc, sf_tauoc )      !* read wave norm stress from external forcing
         tauoc_wave(:,:) = sf_tauoc(1)%fnow(:,:,1)
      ENDIF

      IF ( ln_sdw )  THEN                         !==  Computation of the 3d Stokes Drift  ==! 
         !
         CALL fld_read( kt, nn_fsbc, sf_sd )      !* read wave parameters from external forcing
         swh(:,:)     = sf_sd(jp_swh)%fnow(:,:,1)   ! significant wave height
         wmp(:,:)     = sf_sd(jp_wmp)%fnow(:,:,1)   ! wave mean period
         zusd2dt(:,:) = sf_sd(jp_usd)%fnow(:,:,1)   ! 2D zonal Stokes Drift at T point
         zvsd2dt(:,:) = sf_sd(jp_vsd)%fnow(:,:,1)   ! 2D meridional Stokes Drift at T point
         !
         !==  Computation of the 3d Stokes Drift according to Breivik et al.,2014
         !(DOI: 10.1175/JPO-D-14-0020.1)==! 
         !
         CALL wrk_alloc( jpi,jpj,jpk, ze3hdiv )
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
               ! On T grid
               ! Stokes transport speed estimated from Hs and Tmean
               ztransp = 2.0_wp*rpi*swh(ji,jj)**2.0_wp/(16.0_wp*MAX(wmp(ji,jj),0.0000001_wp))
               ! Stokes surface speed
               zsp0 = SQRT( zusd2dt(ji,jj)**2 + zvsd2dt(ji,jj)**2)
               ! Wavenumber scale
               zk = ABS(zsp0)/MAX(ABS(5.97_wp*ztransp),0.0000001_wp)
               ! Depth attenuation
               zfac(ji,jj) = EXP(-2.0_wp*zk*fsdept(ji,jj,jk))/(1.0_wp+8.0_wp*zk*fsdept(ji,jj,jk))
               END DO
            END DO
            !
            DO jj = 1, jpj
               DO ji = 1, jpi
                 usd3dt(ji,jj,jk) = zfac(ji,jj) * zusd2dt (ji,jj) * tmask(ji,jj,jk)
                 vsd3dt(ji,jj,jk) = zfac(ji,jj) * zvsd2dt (ji,jj) * tmask(ji,jj,jk)
               END DO
            END DO
         END DO
         ! Into the U and V Grid
         DO jk = 1, jpkm1
            DO jj = 1, jpjm1
               DO ji = 1, jpim1
               usd3d(ji,jj,jk) =   0.5 *  umask(ji,jj,jk)  *        &
                               &  (usd3dt(ji,jj,jk) + usd3dt(ji+1,jj,jk))
               vsd3d(ji,jj,jk) =  0.5 *  vmask(ji,jj,jk)  *           &
                               &  (vsd3dt(ji,jj,jk) + vsd3dt(ji,jj+1,jk))
               END DO
            END DO
         END DO
         !
         CALL lbc_lnk( usd3d(:,:,:), 'U', -1. )
         CALL lbc_lnk( vsd3d(:,:,:), 'V', -1. )
         !
         DO jk = 1, jpkm1                       ! e3t * Horizontal divergence
            DO jj = 2, jpjm1
               DO ji = fs_2, fs_jpim1   ! vector opt.
                  ze3hdiv(ji,jj,jk) = (  e2u(ji  ,jj) * fse3u_n(ji  ,jj,jk) * usd3d(ji  ,jj,jk)     &
                     &                 - e2u(ji-1,jj) * fse3u_n(ji-1,jj,jk) * usd3d(ji-1,jj,jk)     &
                     &                 + e1v(ji,jj  ) * fse3v_n(ji,jj  ,jk) * vsd3d(ji,jj  ,jk)     &
                     &                 - e1v(ji,jj-1) * fse3v_n(ji,jj-1,jk) * vsd3d(ji,jj-1,jk)   ) * r1_e1e2t(ji,jj)
               END DO  
            END DO  
            IF( .NOT. AGRIF_Root() ) THEN
               IF( nbondi ==  1 .OR. nbondi == 2 )   ze3hdiv(nlci-1,   :  ,jk) = 0._wp      ! east
               IF( nbondi == -1 .OR. nbondi == 2 )   ze3hdiv(  2   ,   :  ,jk) = 0._wp      ! west
               IF( nbondj ==  1 .OR. nbondj == 2 )   ze3hdiv(  :   ,nlcj-1,jk) = 0._wp      ! north
               IF( nbondj == -1 .OR. nbondj == 2 )   ze3hdiv(  :   ,  2   ,jk) = 0._wp      ! south
            ENDIF
         END DO
         CALL lbc_lnk( ze3hdiv, 'T', 1. ) 
         !
         DO jk = jpkm1, 1, -1                   !* integrate from the bottom the e3t * hor. divergence
            wsd3d(:,:,jk) = wsd3d(:,:,jk+1) - ze3hdiv(:,:,jk)
         END DO
#if defined key_bdy
         IF( lk_bdy ) THEN
            DO jk = 1, jpkm1
               wsd3d(:,:,jk) = wsd3d(:,:,jk) * bdytmask(:,:)
            END DO
         ENDIF
#endif
         CALL wrk_dealloc( jpi,jpj,jpk, ze3hdiv )

         IF ( ln_zdfqiao ) THEN
            CALL fld_read( kt, nn_fsbc, sf_wn )      !* read wave parameters from external forcing
            wnum(:,:) = sf_wn(1)%fnow(:,:,1)
           
            ! Calculate the module of the stokes drift on T grid
            !-------------------------------------------------
            DO jj = 1, jpj
               DO ji = 1, jpi
                   tsd2d(ji,jj) = SQRT( zusd2dt(ji,jj)**2 + zvsd2dt(ji,jj)**2)
               END DO
            END DO
         ENDIF
         !
      ENDIF
      !
   END SUBROUTINE sbc_wave
      
   !!======================================================================
END MODULE sbcwave
